## 介绍
本项目是之前的cobra-api组件代码。

## 接口返回格式修改

由于cobra-api组件中接口返回的格式不统一，造成sc-web调用它时出现数据不一致的问题。现在需要统一cobra-api的API接口返回数据格式，修改方式是：

为每个接口函数新增装饰器：`json_wrapper`，代码如下：

```python
def json_wrapper(func):
    def _func(*args, **kwargs):
        try:
            result = func(*args, **kwargs)

            if isinstance(result, (str, unicode)):
                try:
                    result = json.loads(result)
                except:
                    pass

            if isinstance(result, dict) and 'action' in result:
                return result

            if result is None:
                return {'action': True}
            return {'action': True, 'data': result}
        except:
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            return {'action': False, 'error': str(exc_value)}

    return _func
```


实现效果：
- 当原接口无返回值或返回值为None时，返回`{'action': True}`，表示接口执行成功
- 当原接口有非None返回值时，返回`{'action': True, 'data': result}`，表示接口执行成功，并附带返回数据
- 当原接口执行错误抛出异常时，返回`{'action': False, 'error': str(e)}`，表示接口执行失败，并附带返回错误提示

## 涉及接口类型

使用以上方法修改的涉及几类接口，具体如下。

### 之前cobra-api内部出错返回http code 500（涉及所有接口）

- Before: `http 500`，无错误信息提示
- After: `{'action': False, 'error': 错误提示信息（一般是Exception的字符串表达） }`

### 之前返回dict或者list数据的接口

如：`GET /conf/autodiscovery/jobs`

- Before: `jobs（列表类型数据）`
- After: `{'action': True, 'data': jobs}`

### 之前返回True/False字符串的接口

涉及接口：
- `GET /monitor/checkservicecount` 
- `GET /monitor/checkhostcount`

- Before: `'True' or 'False'`
- After: `{'action': True}` or `{'action': False, 'error': 'check service count failed'}`

### 之前无任何返回值的接口

如：`DELETE /conf/service/<serviceID:int>`

- Before: `None`
- After: {'action': True}

### 之前根据执行命令结果返回的接口

如：`POST /conf/createhostconfigfile`

- Before: `errorResult（错误详情）` or `'0'（命令执行成功标志）` or `'1'等非0数字的字符创形式（命令执行错误标志）`
- After: `{'action': True}` or `{'action': False, error: errorResult（执行命令的加工错误输出） or output（执行命令的原始错误输出）}`

