#!/usr/bin/python
'''
''' 
import os
from  xml.dom import  minidom
from string import atoi
import json
from src.config.db.dboption import Options, table2class
from src.config.service import actionutils
from src.utils import utils

address = ""
exclude_address = ""
result_ips = []
host_list = []

class JobManager():
    '''
    Job Manager, can save jobs, update job state, get all jobs and get job details.
    '''
    def save_job(self, job_range = '', state = 0,  count = 0, filename = ''):
        job_dict = {}
        job_dict['iprange'] = job_range
        job_dict['state'] = state
        job_dict['count'] = count
        job_dict['file_name'] = filename
        jobid = actionutils.insertoneinformation('autodiscovery', job_dict)
        return jobid
    
    def update_job_state(self, jobid, job_state, count):
        key = {'id':jobid}
        update_info = {'state':job_state, 'count': count}
        actionutils.updateoneinformation('autodiscovery', key, update_info)
    
    def get_job_details(self, job_id):
        key_value = {'id':job_id}
        data = actionutils.selectdetailinformation('autodiscovery', key_value)
        if data == []:
            return {'state':-1}
        else:
            jobinfo = json.loads(data)
            joblist = jobinfo['autodiscovery']
            if len(joblist) > 0:
                if joblist[0]['state'] == '1':
                    return {'state':1, 'filename': joblist[0]['file_name']}
                else:
                    return {'state':0}
    
    def get_jobs(self):
        job_data = actionutils.selectall('autodiscovery', 0, 0)
        return job_data
    
class XMLParser():
    '''
    XML parser, can transform xml file to json.
    '''
    def get_attrvalue(self, node, attrname):
        return node.getAttribute(attrname) if node else ''
    
    def get_nodevalue(self, node, index = 0):
        return node.childNodes[index].nodeValue if node else ''
    
    def get_xmlnode(self, node,name):
        return node.getElementsByTagName(name) if node else []
    
    def xml_to_string(self, filename='output.xml'):
        doc = minidom.parse(filename)
        return doc.toxml('UTF-8')
    
    def get_xml_data(self, filename='output.xml'):
        '''
        get xml file content and transform to json
        '''
        doc = minidom.parse(filename) 
        root = doc.documentElement
        host_nodes = self.get_xmlnode(root,'host')
        host_dict = {} 
        for node in host_nodes:
            node_status = self.get_xmlnode(node,'status')
            host_status = self.get_attrvalue(node_status[0], 'state')
            #print 'hoststatus:',host_status
            host_dict['hoststatus'] = str(host_status)
            node_address = self.get_xmlnode(node, 'address')
            for addr in node_address:
                if self.get_attrvalue(addr, 'addrtype') == 'ipv4':
                    address = self.get_attrvalue(addr, 'addr')
                    #print address
                    host_dict['address'] = str(address)
            host_dict['hostname'] = host_dict['address']    
            ports_node = self.get_xmlnode(node, 'ports')
            port_list = []
            for port in self.get_xmlnode(ports_node[0], 'port'):
                port_dict = {}    
                protocol = self.get_attrvalue(port, 'protocol')
                portid = self.get_attrvalue(port, 'portid')
                state = self.get_attrvalue(self.get_xmlnode(port, 'state')[0], 'state')
                service_name = self.get_attrvalue(self.get_xmlnode(port, 'service')[0], 'name')
                #print protocol, portid, state, service_name
                port_dict['name'] = str(service_name)
                port_dict['protocol'] = str(protocol)
                port_dict['port'] = atoi(portid)
                port_list.append(port_dict)
            #print port_list
            host_dict['ports'] = port_list
            #print host_dict
        return host_dict
            
    def test_xmltostring(self):
        '''
        transfom xml file contend to sting and print them.
        '''
        print self.xml_to_string()

def get_alive_ips(ip_range):
    '''
    get alive ips.
    ip_range: ip range.
    return: alive ips
    '''
    alive_ips = []
    cmd = '/usr/sbin/fping -a -r 1 -g '+ip_range+' 2> /dev/null'
    ips = os.popen(cmd).readlines()
    for i in range(len(ips)):
        alive_ips.append(ips[i].strip('\n'))
    return alive_ips

def get_old_ips():
    '''
    get old (already configured) ips.
    ip_range: ip range.
    return: old(configured) ips
    '''
    option = Options()
    old_ips_list = option.selectoneclo(table2class['host'], 'address')
    old_ips = utils.list2list(old_ips_list)
    return old_ips

def get_scan_ips(alive_ips, old_ips):
    '''
    get scan ips.
    ip_range: ip range.
    alive_ips: alive ips
    old_ips: old(configured) ips
    return: scan ips
    '''
    scan_ips = []
    for ip in alive_ips:
        if ip in old_ips:
            pass
        else:
            scan_ips.append(ip)
    return scan_ips

def do_nmap_scans(scan_ips):
    xml_parser = XMLParser()
    filename = '/tmp/temp.xml'
    for ip in scan_ips:
        cmd = '/usr/bin/nmap -oX '+filename+' '+ip
        os.system(cmd)   
        hosts = xml_parser.get_xml_data(filename)
        host_list.append(hosts)
    return json.dumps({'scan_results':host_list})

def do_output(filename, results):
    '''write scan results in file
    filename: file name to write.
    resultes: scan results.
    '''
    file_object = open(filename, 'w')
    file_object.write(results)
    file_object.close()
    return 1
    
def do_jobs(iprange = '', filename = ''):
    '''save a new job.
    iprange: ip range that is used to scan
    filename: file name that save scan results
    '''
    jobmanager = JobManager()
    state = '0'
    count = 0
    jobid = jobmanager.save_job(iprange, state, count, filename)
    jobs = jobmanager.get_jobs()
    return jobs

def get_all_jobs():
    '''
    get all jobs and return all jobs
    '''
    jobmanager = JobManager()
    jobs = jobmanager.get_jobs()
    return jobs
    
def scan_job(jobid, iprange, filename):
    '''scan a job and update job state
    jobid: job id.
    iprange: ip range that is used to scan
    filename: file name that save scan results
    '''
    jobmanager = JobManager()
    alive_ips = get_alive_ips(iprange)
    old_ips = get_old_ips()
    scan_ips = get_scan_ips(alive_ips, old_ips)
    ip_count = len(scan_ips)
    results = do_nmap_scans(scan_ips)
    res = do_output(filename, results)
    if res == 1:
        job_state = '1'
        jobmanager.update_job_state(jobid, job_state, ip_count)
        jobs = jobmanager.get_jobs()
        return jobs
        
def get_job_result(jobid):
    '''get job scan results.
    jobid: job id.
    '''
    #print 'get job detail'
    jobmanager = JobManager()
    result = jobmanager.get_job_details(jobid)
    #print result
    return result
    