'''
Define command and relationinformation

Created on 2012-5-24

@author: cailiuqing
'''


from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation
import time

DeclarativeBase = declarative_base()


class TblCommand(DeclarativeBase):
    __tablename__ = 'tbl_command'

    #column definitions
    access_group = Column('access_group', INTEGER(), nullable=False)
    active = Column('active', Enum(u'0', u'1'), nullable=False)
    command_line = Column('command_line', TEXT(), nullable=False)
    command_name = Column('command_name', VARCHAR(length=255), nullable=False)
    display_name = Column('display_name', VARCHAR(length=255), nullable=False)
    command_description = Column('command_description', TEXT(), nullable=False)
    command_type = Column('command_type', Integer(), nullable=False)
    config_id = Column('config_id', Integer(), nullable=False)
    id = Column('id', INTEGER(), primary_key=True, nullable=False)
    last_modified = Column('last_modified', DATETIME(), nullable=False)
    register = Column('register', Enum(u'0', u'1'), nullable=False)
    is_system_defined = Column('is_system_defined', Integer(), nullable=False)
    
    def __init__(self,command_name='',\
                 command_line='',\
                 display_name='',\
                 command_description='',\
                 command_type=0,\
                 register='1',\
                 active='1',\
                 last_modified=time.strftime('%Y-%m-%d %H:%M:%S',time.localtime()),\
                 access_group=0,\
                 config_id=1,\
                 is_system_defined=1):
        self.command_name = command_name
        self.command_line = command_line
        self.display_name = display_name
        self.command_description = command_description
        self.command_type = command_type
        self.register = register
        self.active = active
        self.last_modified = last_modified
        self.access_group = access_group
        self.config_id = config_id
        self.is_system_defined = is_system_defined
    
    def __repr__(self):
        return "{'id':%s,\
            'display_name':%s,\
            'command_name':%s,\
            'command_line':%s,\
            'command_description':%s,\
            'command_type':%s,\
            'register':%s,\
            'active':%s,\
            'last_modified':%s,\
            'access_group':%s,\
            'config_id':%s,\
            'is_system_defined':%s}" % (self.id,\
            self.display_name,\
            self.command_name,\
            self.command_line,\
            self.command_description,\
            self.command_type,\
            self.register,\
            self.active,\
            self.last_modified,\
            self.access_group,\
            self.config_id,\
            self.is_system_defined)

class TblRelationinformation(DeclarativeBase):
    __tablename__ = 'tbl_relationinformation'

    #column definitions
    fieldName = Column('fieldName', VARCHAR(length=255), nullable=False)
    flags = Column('flags', VARCHAR(length=255), nullable=False)
    fullRelation = Column('fullRelation', Integer(), nullable=False)
    id = Column('id', INTEGER(), primary_key=True, nullable=False)
    linkTable = Column('linkTable', VARCHAR(length=255), nullable=False)
    master = Column('master', VARCHAR(length=255), nullable=False)
    tableName1 = Column('tableName1', VARCHAR(length=255), nullable=False)
    tableName2 = Column('tableName2', VARCHAR(length=255), nullable=False)
    target1 = Column('target1', VARCHAR(length=255), nullable=False)
    target2 = Column('target2', VARCHAR(length=255), nullable=False)
    targetKey = Column('targetKey', VARCHAR(length=255), nullable=False)
    type = Column('type', Integer(), nullable=False)
    
    def __init__(self,master='',\
                tableName1='',\
                tableName2='',\
                fieldName='',\
                linkTable='',\
                target1='',\
                target2='',\
                targetKey='',\
                fullRelation='0',
                flags='',\
                type=''):
        self.master = master
        self.tableName1 = tableName1
        self.tableName2 = tableName2
        self.fieldName = fieldName
        self.linkTable = linkTable
        self.target1 = target1
        self.target2 = target2
        self.targetKey = targetKey
        self.fullRelation = fullRelation
        self.flags = flags
        self.type = type
    
    def __repr__(self):
        return "{`id`:%s,\
            `master`:%s,\
            `tableName1`:%s,\
            `tableName2`:%s,\
            `fieldName`:%s,\
            `linkTable`:%s,\
            `target1`:%s,\
            `target2`:%s,\
            `targetKey`:%s,\
            `fullRelation`:%s,\
            `flags`:%s,\
            `type`:%s}" % (self.id,\
            self.master,\
            self.tableName1,\
            self.tableName2,\
            self.fieldName,\
            self.linkTable,\
            self.target1,\
            self.target2,\
            self.targetKey,\
            self.fullRelation,\
            self.flags,\
            self.type)
    
class TblNotificationInfo(DeclarativeBase):
    __tablename__ = 'tbl_notification_info'
    id = Column("id", Integer, primary_key=True,autoincrement=True,nullable=False)
    detail = Column("detail", Text)
    year = Column("year", Integer)
    month = Column("month", Integer)
    day = Column("day", Integer)
    time = Column("time",  DATETIME())
    level = Column("level", String(10))
