'''
Define timedefinition and timeperiod

Created on 2012-5-24

@author: cailiuqing
'''

from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation
import time

DeclarativeBase = declarative_base()

class TblTimedefinition(DeclarativeBase):
    __tablename__ = 'tbl_timedefinition'

    #column definitions
    definition = Column('definition', VARCHAR(length=255), nullable=False)
    id = Column('id', INTEGER(), primary_key=True, nullable=False,autoincrement=True)
    last_modified = Column('last_modified', DATETIME(), nullable=False)
    range = Column('range', TEXT(), nullable=False)
    tipId = Column('tipId', INTEGER(), nullable=False)

    #relation definitions
    
    def __init__(self,tipId='',\
                definition='',\
                range='',\
                last_modified=time.strftime('%Y-%m-%d %H:%M:%S',time.localtime())):
        self.tipId = tipId
        self.definition = definition
        self.range = range
        self.last_modified = last_modified
        
    def __repr__(self):
        return "{`id`:%s,\
            `tipId`:%s,\
            `definition`:%s,\
            `range`:%s,\
            `last_modified`:%s}" % (self.id,\
            self.tipId,\
            self.definition,\
            self.range,\
            self.last_modified)


class TblTimeperiod(DeclarativeBase):
    __tablename__ = 'tbl_timeperiod'

    #column definitions
    access_group = Column('access_group', INTEGER(), nullable=False)
    active = Column('active', Enum(u'0', u'1'), nullable=False)
    alias = Column('alias', VARCHAR(length=255), nullable=False)
    config_id = Column('config_id', Integer(), nullable=False)
    exclude = Column('exclude', Integer(), nullable=False)
    id = Column('id', INTEGER(), primary_key=True, nullable=False)
    last_modified = Column('last_modified', DATETIME(), nullable=False)
    name = Column('name', VARCHAR(length=255), nullable=False)
    register = Column('register', Enum(u'0', u'1'), nullable=False)
    timeperiod_name = Column('timeperiod_name', VARCHAR(length=255), nullable=False)
    use_template = Column('use_template', Integer(), nullable=False)

    #relation definitions
    
    def __init__(self,timeperiod_name='',\
                alias='',\
                exclude='0',\
                use_template='0',\
                name='',\
                register='1',
                active='1',
                last_modified=time.strftime('%Y-%m-%d %H:%M:%S',time.localtime()),\
                access_group='0',
                config_id='0',):
               
        self.timeperiod_name = timeperiod_name
        self.alias = alias
        self.exclude = exclude
        self.use_template = use_template
        self.name = name
        self.register = register
        self.active = active
        self.last_modified = last_modified
        self.access_group = access_group
        self.config_id = config_id
    
    def __repr__(self):
        return "{`id`:%s,\
                `timeperiod_name`:%s,\
                `alias`:%s,\
                `exclude`:%s,\
                `use_template`:%s,\
                `name`:%s,\
                `register`:%s,\
                `active`:%s,\
                `last_modified`:%s,\
                `access_group`:%s,\
                `config_id`:%s}" % (self.id,\
                self.timeperiod_name,\
                self.alias,\
                self.exclude,\
                self.use_template,\
                self.name,\
                self.register,\
                self.active,\
                self.last_modified,\
                self.access_group,\
                self.config_id)
