import commands
import re
import string
import json
import time


def switch_configwizard_get_cfgmaker_cmd(snmpcommunity,address,snmpversion="1",defaultspeed="100000000"):
    '''
        Get port detail information by command 'cfgmaker'
    '''
    cmd = "cfgmaker --show-op-down --zero-speed="+defaultspeed+" --snmp-options=:::::"+snmpversion+" --noreversedns "+snmpcommunity+"@"+address
    #print cmd
    (status, output) = commands.getstatusoutput(cmd)
    outfile = '/opt/cobra/mrtgscan-'+address
    f = file(outfile,'w+')
    f.write(output)
    #print "The end"
    f.close()
    #outfile_tmp = '/opt/cobra/mrtgscan-10.1.83.200'
    #print outfile_tmp,address
    #print output
    if output.find("SNMP Error") != -1:
        #print "SNMP no response.Couldn't monitor this switch."
        return "SNMP no response.Couldn't monitor this switch."
    else:
        switch_configwizard_add_walk_file_to_mrtg(outfile,address)
    return outfile
    

def switch_configwizard_get_readable_port_line_speed(max_bytes):
    '''
        Change the speed to readable port line speed
    '''
    bps = max_bytes*8    
    kbps = bps/1000
    mbps = bps/1000/1000
    gbps = bps/1000/1000/1000
    
    if gbps >= 1:
        speed = gbps
        label = "Gbps"
    elif mbps >= 1:
        speed = mbps
        label = "Mbps"
    elif kbps >= 1 :
        speed = kbps
        label = "Kbps"
    else:
        speed=bps+" bps"
        label="bps"
    output= str(speed)+" "+label
    #print output
    return output

def switch_configwizard_read_walk_file(f,address):
    pass

def switch_configwizard_add_walk_file_to_mrtg(f,address):
    #debug = False
    mrtg_cfg = "/etc/mrtg/mrtg.cfg"
    logformat = "LogFormat: rrdtool\n"
    workdir = "WorkDir: /var/lib/mrtg\n"
    # open the mrtg file for reading - see if the switch is already in there
    already_in_mrtg = False
    already_has_logformat = False
    already_has_workdir = False
    fo = open(mrtg_cfg,"r")
    if not fo:
        return "UNABLE TO OPEN "+mrtg_cfg+" FOR READING!"
        #if debug:
            #return "UNABLE TO OPEN "+mrtg_cfg+" FOR READING!"
    #if debug:
    #else :
        #return "HAVE "+mrtg_cfg+" OPEN..."

    for s in fo.readlines():
        if re.findall("LogFormat",s):
            already_has_logformat = True
            #print "HAS LOGFORMAT"
        if re.findall("WorkDir",s):
            already_has_workdir = True
        if re.findall("#Target",s):
            continue
        if re.findall("Target\[",s) and re.findall(address,s):
            already_in_mrtg = True
            #print "ALREADY IN MRTG!"+address

    fo.close()

    #already in mrtg so bail...
    if already_in_mrtg:
        #return True
        print "Aleady has this target"
        return True

    #open the mrtg file for appending
    fo = open(mrtg_cfg,"a+")
    if not fo:
    #   if debug :
        print "UNABLE TO OPEN "+mrtg_cfg+" FOR APPENDING!"
        return False
    
    #already has logformat and workdir setting
    if not already_has_logformat and not already_has_workdir:
        fo.write(logformat)
        fo.write(workdir)
        
    #open the walk file for reading
    fi = open(f,"r")
    if fi:
        nowtime = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
        #str1 = str(fo.name)+
        str1 = "\n\n#### ADDED BY COBRA (USER: icinga, DATE: "+str(nowtime)+") ####\n"
        fo.write(str1)
        for s in fi.readlines():
            #print s
            fo.write(s)
    else:
        return "UNABLE TO OPEN "+f+" FOR READING!"

    fo.close()
    
    #immediately tell mrtg to generate data from the new walk file 
    #if we didn't do this, cobra might send alerts about missing rrd files!
    cmd="env LANG=C /usr/bin/mrtg "+mrtg_cfg+" &"
    commands.getstatusoutput(cmd)
    return True
    
def switch_configwizard_func(snmpcommunity,address,snmpversion,defaultspeed):
    outfile = switch_configwizard_get_cfgmaker_cmd(snmpcommunity,address,snmpversion,defaultspeed)
    #outfile_tmp = '/opt/cobra/mrtgscan-10.1.83.200'
    file = open(outfile,'r')
    #print type(outfile)
    #print type(file)
    #print file.name
    ports = []
    port = {}
    port_count = 0
    for s in file.readlines():
        # found the target line (contains port number)   
        po = re.findall("Target",s)
        if len(po) >0:
            port_count = port_count + po.count("Target")
        if re.findall("# Target",s):
            continue
        elif re.findall("Target",s):
            port["port_number"] = s.split(":")[1].replace(' ','')
            port["port_description"] = "Port "+s.split(":")[1]
        # get the max bytes of this port
        elif re.findall("MaxBytes",s):
            bytes = s.split(":")[1].replace("\n","")
            value = switch_configwizard_get_readable_port_line_speed(string.atoi(bytes))
            port["max_bytes"] = value
        # get the detail description of this port
        elif re.findall("MRTG_INT_DESCR=",s):
            port["port_long_description"] = s.split("=")[2].replace("\n","").replace('"','')
        if len(port) == 4:
            ports.append(port)
            port = {}
        #elif port == {}:
        #    return "ERROR:Please make sure the system has cfgmaker or the target switch is alive."     
    file.close()
    if port_count == 0:
        return "ERROR:Please make sure the system has cfgmaker or the target switch is alive."     
    tblclass = 'switches'
    data = {"port_counts":port_count}
    data[tblclass] = ports
    encodedata = json.dumps(data,ensure_ascii=False)
    return encodedata

#print switch_configwizard_func('public','10.1.83.200','','')
#switch_configwizard_get_readable_port_line_speed(12500000)
#address = '10.1.83.200'
#outfile = '/opt/cobra/mrtgscan-'+address
#switch_configwizard_add_walk_file_to_mrtg(outfile,address)
#switch_configwizard_get_cfgmaker_cmd('public','10.1.83.202')

