'''
Created on 2012-7-4

@author: TianPengfei
'''
from sqlalchemy.dialects.mysql import BIGINT, DATETIME, DOUBLE, INTEGER, SMALLINT, TEXT, VARCHAR
from sqlalchemy import Column
from sqlalchemy import BigInteger, Float, String
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.schema import UniqueConstraint
from sqlalchemy.schema import Index
from sqlalchemy import func,or_
import ConfigParser

cfg_file = "/opt/cobra/cobra-api/config.conf"
#cfg_file = "/root/workspace/Cobra/code/cobra/config.conf"

conf = ConfigParser.ConfigParser()
conf.read(cfg_file)
#print conf.sections()
db_host = conf.get("db", "db_host")
db_port = conf.get("db", "db_port")
db_user = conf.get("db", "db_user")
db_pass = conf.get("db", "db_pass")
db_name = conf.get("db", "db_name")

connect = 'postgresql://'+db_user+':'+db_pass+'@'+db_host+':'+db_port+'/'+db_name+'?charset=utf8'

mysql_engine = create_engine(connect, echo=False, pool_size = 20,pool_recycle=3600)
base = declarative_base()
metadata = base.metadata
metadata.bind = mysql_engine

class IcingaLogentries(base):
        __tablename__ = 'icinga_logentries'
        
        logentry_id = Column(BIGINT(20),primary_key=True,autoincrement=True)
        instance_id = Column(BIGINT(20),unique=True,default = 0)
        logentry_time = Column(DATETIME,default = '0000-00-00 00:00:00')
        entry_time = Column(DATETIME,default = '0000-00-00 00:00:00')
        entry_time_usec = Column(INTEGER(11),default = 0)
        logentry_type = Column(INTEGER(11), default = 0)
        logentry_data = Column(VARCHAR(255),default = '')
        realtime_data = Column(SMALLINT(6),default = 0)
        inferred_data_extracted = Column(SMALLINT(6),default = 0)
        
        def __init__(self, instance_id=0,logentry_time='0000-00-00 00:00:00', \
                     entry_time='0000-00-00 00:00:00', entry_time_usec=0, logentry_type=0, \
                     logentry_data='', realtime_data=0, inferred_data_extracted=0):
            
            self.instance_id = instance_id
            self.logentry_time = logentry_time
            self.entry_time = entry_time
            self.entry_time_usec = entry_time_usec
            self.logentry_type = logentry_type
            self.logentry_data = logentry_data
            self.realtime_data = realtime_data
            self.inferred_data_extracted = inferred_data_extracted
            
            
        def __repr__(self):
            
                return "{'logentry_id':%d, \
                         'instance_id':%d, \
                         'logentry_time':%s, \
                         'entry_time':%s, \
                         'entry_time_usec':%d, \
                         'logentry_type':%d \
                         'logentry_data':%s \
                         'realtime_data':%s \
                         'inferred_data_extracted':%d \
                         }" % (self.logentry_id, \
                         self.instance_id, \
                         self.logentry_time, \
                         self.entry_time, \
                         self.entry_time_usec, \
                         self.logentry_type, \
                         self.logentry_data, \
                         self.realtime_data, \
                         self.inferred_data_extracted)


class IcingaLogentriesMapping(base):
        __tablename__ = 'icinga_logentrytypemapping'
        
        logentry_type_id = Column(INTEGER(11),primary_key=True,autoincrement=True)
        logentry_type_num = Column(INTEGER(11))
        logentry_type_name = Column(VARCHAR(11))
        
        def __init__(self , logentry_type_num = 0 , logentry_type_name = ""):
            
            self.logentry_type_num = logentry_type_num
            self.logentry_type_name = logentry_type_name
            
            
        def __repr__(self):
            
            return "{'logentry_type_id':%d \
                     'logentry_type_num':%d \
                     'logentry_type_name':%d \
                    }"% (self.logentry_type_id, \
                    self.logentry_type_num, \
                    self.logentry_type_name)

def get_paging_logentries(limit,start):
    '''
    Get logentries information by paging
    '''
    Session = sessionmaker(bind=mysql_engine)
    session = Session()
    if limit == 0 and start == 0:
        result = session.query(IcingaLogentries.logentry_time,IcingaLogentriesMapping.logentry_type_name,IcingaLogentries.logentry_data) \
                        .filter(IcingaLogentries.logentry_type == IcingaLogentriesMapping.logentry_type_num).all() 
    else:
        result = session.query(IcingaLogentries.logentry_time,IcingaLogentriesMapping.logentry_type_name,IcingaLogentries.logentry_data) \
                        .filter(IcingaLogentries.logentry_type == IcingaLogentriesMapping.logentry_type_num)\
                        .order_by(IcingaLogentries.logentry_time.desc())\
                        .slice(start,start+limit).all()
    session.flush()
    session.close()
    return result     

def get_all_logentries_count():
    '''
    Get all logentries information
    '''
    Session = sessionmaker(bind=mysql_engine)
    session = Session()
    count = session.query(IcingaLogentries).count()
    session.flush()
    session.close()
    return count  

def get_paging_logentries_by_keyword(limit,start,keyword):
    '''
    Get logentries by keyword
    '''
    Session = sessionmaker(bind=mysql_engine)
    session = Session()
    if limit == 0 and start == 0:
        result = session.query(IcingaLogentries.logentry_time,IcingaLogentriesMapping.logentry_type_name, IcingaLogentries.logentry_data) \
                    .filter(IcingaLogentries.logentry_type == IcingaLogentriesMapping.logentry_type_num) \
                    .filter(or_(IcingaLogentries.logentry_time.like('%'+("%s")%keyword+'%'), \
                                IcingaLogentriesMapping.logentry_type_name.like('%'+("%s")%keyword+'%'), \
                                IcingaLogentries.logentry_data.like('%'+("%s")%keyword+'%'))
                            ).all()
    else:
        result = session.query(IcingaLogentries.logentry_time,IcingaLogentriesMapping.logentry_type_name, IcingaLogentries.logentry_data) \
                        .filter(IcingaLogentries.logentry_type == IcingaLogentriesMapping.logentry_type_num) \
                        .filter(or_(IcingaLogentries.logentry_time.like('%'+("%s")%keyword+'%'), \
                                    IcingaLogentriesMapping.logentry_type_name.like('%'+("%s")%keyword+'%'), \
                                    IcingaLogentries.logentry_data.like('%'+("%s")%keyword+'%'))
                                ).slice(start,start+limit)
    session.flush()
    session.close()

    return result

def get_all_logentries_by_keyword_count(keyword):
    '''
    Get count of all logentries by keyword
    '''
    Session = sessionmaker(bind=mysql_engine)
    session = Session()
    count = session.query(IcingaLogentries.logentry_time,IcingaLogentriesMapping.logentry_type_name, IcingaLogentries.logentry_data) \
                    .filter(IcingaLogentries.logentry_type == IcingaLogentriesMapping.logentry_type_num) \
                    .filter(or_(IcingaLogentries.logentry_time.like('%'+("%s")%keyword+'%'), \
                                IcingaLogentriesMapping.logentry_type_name.like('%'+("%s")%keyword+'%'), \
                                IcingaLogentries.logentry_data.like('%'+("%s")%keyword+'%'))
                            ).count()
    session.flush()
    session.close()
    return count
